/************************************************************************
 *   IRC - Internet Relay Chat, src/m_ungline.c
 *   Copyright (C) 1990 Jarkko Oikarinen and
 *                      University of Oulu, Computing Center
 *
 *   See file AUTHORS in IRC package for additional names of
 *   the programmers. 
 *
 *   This program is free software; you can redistribute it and/or modify
 *   it under the terms of the GNU General Public License as published by
 *   the Free Software Foundation; either version 1, or (at your option)
 *   any later version.
 *
 *   This program is distributed in the hope that it will be useful,
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *   GNU General Public License for more details.
 *
 *   You should have received a copy of the GNU General Public License
 *   along with this program; if not, write to the Free Software
 *   Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 *
 *
 *   $Id: m_ungline.c,v 1.37 2000/06/10 02:45:36 decho Exp $
 */
#include "m_commands.h"
#include "channel.h"
#include "client.h"
#include "common.h"
#include "dline_conf.h"
#include "fileio.h"
#include "irc_string.h"
#include "ircd.h"
#include "mtrie_conf.h"
#include "numeric.h"
#include "s_conf.h"
#include "s_log.h"
#include "s_misc.h"
#include "send.h"
#include "struct.h"

#include <stdio.h>
#include <time.h>
#include <string.h>
#include <unistd.h>

#ifdef GLINES

extern ConfigFileEntryType ConfigFileEntry; 	
extern int    remove_gline_match(char *host,char *user);

/*
** m_ungline
** Added Jul 25, 2000 
**
**      parv[0] = sender
**      parv[1] = address to remove
*
*
* -decho
*
*
*/
int m_ungline (aClient *cptr,aClient *sptr,int parc,char *parv[])
{
  FBFILE* in;
  FBFILE* out;
  int   pairme = NO;
  char  buf[BUFSIZE];
  char  buff[BUFSIZE];  /* matches line definition in s_conf.c */
  char  temppath[256];

  char  *user,*host;
  char  *p;
  int   error_on_write = NO;
  mode_t oldumask;

  if (check_registered(sptr))
    {
      return -1;
    }

  if (!IsAnOper(sptr))  
    {
      sendto_one(sptr, form_str(ERR_NOPRIVILEGES), me.name, 
                 parv[0]);
      return 0;
    }

  if (!IsSetOperUnkline(sptr))
    {
      sendto_one(sptr,":%s NOTICE %s :You have no U flag",me.name,parv[0]);
      return 0;
    }

  if ( parc < 2 )
    {
      sendto_one(sptr, form_str(ERR_NEEDMOREPARAMS),
                 me.name, parv[0], "UNGLINE");
      return 0;
    }

  if ( (host = strchr(parv[1], '@')) || *parv[1] == '*' )
    {
      // Explicit user@host mask given 

      if(host)                  
        {
          user = parv[1];       // here is user part 
          *(host++) = '\0';     // and now here is host 
        }
      else
        {
          user = "*";           // no @ found, assume its *@somehost 
          host = parv[1];
        }
    }
  else
    {
      sendto_one(sptr, ":%s NOTICE %s :Invalid parameters",
                 me.name, parv[0]);
      return 0;
    }

  if( (user[0] == '*') && (user[1] == '\0')
      && (host[0] == '*') && (host[1] == '\0') )
    {
      sendto_one(sptr, ":%s NOTICE %s :Cannot UNG-Line everyone",
                 me.name, parv[0]);
      return 0;
    }



  if(remove_gline_match(host,user))
    {
      sendto_one(sptr, ":%s NOTICE %s :Un-glined [%s@%s] from g-lines",
                 me.name, parv[0],user, host);
      sendto_ops("%s has removed the G-Line for: [%s@%s]",
                 parv[0], user, host );

      log(L_NOTICE, "%s removed G-Line for [%s@%s]",
          parv[0], user, host);
      return 0;
    }
  else
    {
     sendto_one(sptr, ":%s NOTICE %s :Can't find gline for [%s@%s]",
                 me.name, parv[0],user, host);
    }

 return 0;
}

#endif /* GLINES */
